### This calculates the summary statistics of the SC from xeric reference sites

# Clear working space
rm(list = ls())

# Load packages
library(tidyverse)
library(lubridate)
library(readxl)

# Load the data
dat <- read_excel("../_Datasets/Xeric_Reference_Data.xlsx")

# Filter sample size >=6 based on COMID
dat2 <- dat %>%
  group_by(COMID) %>%
  filter(n() >= 6) %>%
  ungroup()

# Calculate the log SC
dat3 <- dat2 %>%
  rename(SC = `uS/cm`) %>%
  mutate(log10_SC = log10(SC))

# Calculate summary statistics
dat4 <- dat3 %>%
  group_by(COMID) %>%
  summarize(Sample_Size = n(),
            Mean = mean(SC),
            Min = min(SC),
            Max = max(SC),
            SD = sd(SC),
            Mean_log10 = mean(log10_SC),
            Min_log10 = min(log10_SC),
            Max_log10 = max(log10_SC),
            SD_log10 = sd(log10_SC)) %>%
  mutate(Range = Max - Min, Range_log10 = Max_log10 - Min_log10) %>%
  select(COMID, Sample_Size, Mean, Min, Max, SD, Range, Mean_log10, Min_log10, Max_log10, SD_log10, Range_log10) %>%
  ungroup()

dat5 <- dat4 %>%
  summarize(across(.fns = mean))

write_csv(dat4, "../_Tables_Figures/Tables/Table_A8.csv")
